﻿Socialist = LibStub("AceAddon-3.0"):NewAddon("Socialist")

--------------------------------------------------------------------------------
--                            VARIABLE INITIALIZE                             --
--------------------------------------------------------------------------------

Socialist.initialized = false

local database_defaults = {
	realm = {
		whodb = {
			["next id"] = 1,
		},
		groupdb = {
			["next id"] = 1,
		},
		profdb = {
			["next id"] = 1,
		},
	},
	profile = {

	},
}

--------------------------------------------------------------------------------
--                                   SETUP                                    --
--------------------------------------------------------------------------------

function Socialist:OnInitialize()
	self.db = LibStub("AceDB-3.0"):New("SocialistDB", database_defaults,
	    true)
	self.whodb = self.db.realm.whodb
	self.groupdb = self.db.realm.groupdb
	self.profdb = self.db.realm.profdb

	self.initialized = true
end

function Socialist:OnEnable()

end

--------------------------------------------------------------------------------
--                                    API                                     --
--------------------------------------------------------------------------------


--[[ Miscellaneous Functions ]]--

function Socialist:ClearDB(database)
	if not self.initialized then return end
	if not database == "whodb" and
	   not database == "groupdb" and
	   not database == "profdb" then return end
	self.db.realm[database] = {}
end

--[[ WhoDB Functions ]]--

function Socialist:WhoDB_GetRecordByID(id)
	if not self.initialized then return end
	if not type(id) == "number" then return end
	if self.whodb["next id"] <= id then return end
	for k,v in ipairs(self.whodb) do
		if v.id == id then
			return v
		end
	end
	return false
end

function Socialist:WhoDB_Query(query,sort,order)
	if not self.initialized then return end
	if not ValidateWhoDBQuery(query) then return end
	local resultset = WhoDB_GatherResultSet(query)
	if sort and (
	    sort == "name" or sort == "guild" or sort == "level" or
	    sort == "class" or sort == "race" or sort == "lastseen" or
	    sort == "zone"
	) then
		-- Ensure order is "DESCENDING" (default) or "ASCENDING"
		order = (order == "ASCENDING" and order) or "DESCENDING"
		sort(resultset,whodb_sorts[sort][order])
	end
	return resultset
end

function Socialist:WhoDB_Modify(character,zone,info)
	if not self.initialized then return end
	if (not character) or (not zone) then return end
	local record = nil
	if type(character) == "number" then
		for k,v in ipairs(self.whodb) do
			if v.id == character then
				record = v
				break
			end
		end
	elseif type(character) == "string" then
		for k,v in ipairs(self.whodb) do
			if v.name == character then
				record = v
				break
			end
		end
		-- If no record exists, create one
		if not record then
			record = {
				["id"] = self.whodb["next id"],
				["name"] = character,
				["normalname"] = Normalize(character),
			}
			tinsert(self.whodb, record)
			self.whodb["next id"] = self.whodb["next id"] + 1
		end
	end
	-- record now points to the record OR is nil if no record exists.
	if record then
		record.lastseen = time()
		record.zone = zone
		for k,v in pairs(info) do
			record[k] = v
		end
	end
	-- return the record's id, or "false" if no record was found or created.
	return record and record.id
end

function Socialist:WhoDB_Remove(id)
	if not self.initialized then return end
	if not id or type(id) ~= "number" then return end
	for k,v in ipairs(self.whodb) do
		if v.id == id then
			tremove(self.whodb,k)
			return true
		end
	end
	return false
end

--[[ GroupDB Functions ]]--
function Socialist:GroupDB_GetRecordByID(groupid)
	if not self.initialized then return end
end

function Socialist:GroupDB_Query(query)
	if not self.initialized then return end

end

function Socialist:GroupDB_Modify(groupid,info)
	if not self.initialized then return end

end

function Socialist:GroupDB_CreateNewGroup()
	if not self.initialized then return end

end

function Socialist:GroupDB_Remove(groupid)
	if not self.initialized then return end

end

function Socialist:GroupDB_AddPlayer(groupid,id)
	if not self.initialized then return end

end

function Socialist:GroupDB_AddRaidID(groupid,raidid)
	if not self.initialized then return end

end

function Socialist:GroupDB_AddInstance(groupid,instance)
	if not self.initialized then return end

end

--[[ ProfDB Functions ]]--

function Socialist:ProfDB_GetRecordByID(profid)
	if not self.initialized then return end

end

function Socialist:ProfDB_Query(query,sort,order)
	if not self.initialized then return end

end

function Socialist:ProfDB_Modify(profid,info)
	if not self.initialized then return end

end

function Socialist:ProfDB_Remove(profid)
	if not self.initialized then return end

end

function Socialist:ProfDB_CreateNewRecipe(name,info)
	if not self.initialized then return end

end

function Socialist:ProfDB_AddIngredient(profid,name,amount)
	if not self.initialized then return end

end

function Socialist:ProfDB_AddCrafter(profid,id)
	if not self.initialized then return end

end

--------------------------------------------------------------------------------
--                            Sorting Functions                               --
--------------------------------------------------------------------------------
local whodb_sorts = {
	["name"] = {
		["DESCENDING"] = function(a,b) return a.name < b.name end,
		["ASCENDING"] = function(a,b) return a.name > b.name end,
	},
	["guild"] = {
		["DESCENDING"] = function(a,b) return a.guild < b.guild end,
		["ASCENDING"] = function(a,b) return a.guild > b.guild end,
	},
	["level"] = {
		["DESCENDING"] = function(a,b) return a.level < b.level end,
		["ASCENDING"] = function(a,b) return a.level > b.level end,
	},
	["class"] = {
		["DESCENDING"] = function(a,b) return a.class < b.class end,
		["ASCENDING"] = function(a,b) return a.class > b.class end,
	},
	["race"] = {
		["DESCENDING"] = function(a,b) return a.race < b.race end,
		["ASCENDING"] = function(a,b) return a.race > b.race end,
	},
	["lastseen"] = {
		["DESCENDING"] = function(a,b)
		    return a.lastseen < b.lastseen end,
		["ASCENDING"] = function(a,b)
		    return a.lastseen > b.lastseen end,
	},
	["zone"] = {
		["DESCENDING"] = function(a,b) return a.zone < b.zone end,
		["ASCENDING"] = function(a,b) return a.zone > b.zone end,
	},
}


--------------------------------------------------------------------------------
--                            Helper Functions                                --
--------------------------------------------------------------------------------

local function ValidateWhoDBQuery(query)
	if not query then return false end
	if query.name and type(query.name) ~= "string" then return false end
	if query.guild and type(query.guild) ~= "string" then return false end
	if query.level and type(query.level) ~= "number" then return false end
	if query.class and type(query.class) ~= "string" then return false end
	if query.race and type(query.race) ~= "string" then return false end
	if query.lastseen and type(query.lastseen) ~= "number" then return false
	    end
	if query.zone and type(query.zone) ~= "string" then return false end
end

local function WhoDB_GatherResultSet(query)
	local result = {}
	for k,v in ipairs(Socialist.whodb) do
		local hit = true
		if hit and query.lastseen then
			hit = query.lastseen >= v.lastseen and hit or false
		end
		if hit and (query.levellow or query.levelhigh) then
			query.levelhigh = query.levelhigh or 80
			query.levellow = query.levellow or 0
			hit = query.levelhigh >= v.level and query.levellow <= v.level and
			    hit or false
		end
		if hit and query.class then
			hit = strfind(strupper(v.class),strupper(query.class)) and
			    hit or false
		end
		if hit and query.race then
			hit = strfind(strupper(v.race),strupper(query.race)) and
			    hit or false
		end
		if hit and query.zone then
			hit = strfind(strupper(v.zone),strupper(query.zone)) and
			    hit or false
		end
		if hit and query.guild then
			hit = strfind(strupper(v.guild),strupper(query.guild)) and
			    hit or strfind(strupper(v.normalguild),strupper(query.guild))
		end
		if hit and query.name then
			hit = strfind(strupper(v.name),strupper(query.name)) and
			    hit or strfind(strupper(v.normalname),strupper(query.name))
		end
		-- Pruning complete
		if hit then
			tinsert(result,v)
		end
	end
	return result
end

local function Normalize(input)
	-- For now, do nothing.
	return input
end



